﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 소켓
        /// </summary>
        private Socket mainSocket;

        /// <summary>
        /// 데이터 바이트 배열
        /// </summary>
        private byte[] dataByteArray = new byte[4096];

        /// <summary>
        /// 캡처 계속 여부
        /// </summary>
        private bool continueCapturing = false;

        /// <summary>
        /// 패킷 번호
        /// </summary>
        private int packetNumber = 1;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이벤트를 설정한다.

            Load                   += Form_Load;
            this.startButton.Click += startButton_Click;
            this.stopButton.Click  += stopButton_Click;
            this.listView.Click    += listView_Click;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            this.packetNumberComboBox.Text = "100";
            
            string ipAddressString = null;

            IPHostEntry ipHostEntry = Dns.GetHostEntry((Dns.GetHostName()));

            if(ipHostEntry.AddressList.Length > 0)
            {
                foreach(IPAddress ipAddress in ipHostEntry.AddressList)
                {
                    ipAddressString = ipAddress.ToString();

                    this.ipAddressComboBox.Items.Add(ipAddressString);
                }
            }       
        }

        #endregion
        #region 시작 버튼 클릭시 처리하기 - startButton_Click(sender, e)

        /// <summary>
        /// 시작 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void startButton_Click(object sender, EventArgs e)
        {
            if(this.ipAddressComboBox.Text == string.Empty)
            {
                MessageBox.Show
                (
                    this,
                    "캡처할 네트워크 인터페이스를 선택해 주시기 바랍니다.",
                    "INFORMATION",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                );

                return;
            }

            try
            {
                if(!this.continueCapturing)
                {
                    this.startButton.Enabled = false;
                    this.stopButton.Enabled  = true;

                    this.continueCapturing = true;

                    this.mainSocket = new Socket(AddressFamily.InterNetwork, SocketType.Raw, ProtocolType.IP);

                    this.mainSocket.Bind(new IPEndPoint(IPAddress.Parse(this.ipAddressComboBox.Text), 0));

                    this.mainSocket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.HeaderIncluded, true);

                    byte[] trueByteArray = new byte[4] { 1, 0, 0, 0 };
                    byte[] outByteArray  = new byte[4] { 1, 0, 0, 0 };

                    this.mainSocket.IOControl(IOControlCode.ReceiveAll, trueByteArray, outByteArray);

                    this.mainSocket.BeginReceive
                    (
                        this.dataByteArray,
                        0,
                        this.dataByteArray.Length,
                        SocketFlags.None,
                        new AsyncCallback(ReceiveCallback),
                        null
                    );
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    this,
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
        #region 중단 버튼 클릭시 처리하기 - stopButton_Click(sender, e)

        /// <summary>
        /// 중단 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.startButton.Enabled = true;
            this.stopButton.Enabled  = false;
            this.continueCapturing   = false;

            this.mainSocket.Close();
        }

        #endregion
        #region 리스트 뷰 클릭시 처리하기 - listView_Click(sender, e)

        /// <summary>
        /// 리스트 뷰 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listView_Click(object sender, EventArgs e)
        {
            this.treeView.Nodes.Clear();

            this.treeView.Nodes.Add(this.listView.SelectedItems[0].SubItems[6].Text);

            for(int n = 7 ; n < 19 ;n++ )
            {
                 this.treeView.Nodes[0].Nodes.Add(this.listView.SelectedItems[0].SubItems[n].Text);
            }

            if(this.listView.SelectedItems[0].SubItems[4].Text == "TCP")
            {
                this.treeView.Nodes.Add(this.listView.SelectedItems[0].SubItems[19].Text);

                for(int n = 20; n < 28; n++)
                {
                    this.treeView.Nodes[1].Nodes.Add(this.listView.SelectedItems[0].SubItems[n].Text);
                }
            }
            else if(this.listView.SelectedItems[0].SubItems[4].Text == "UDP")
            {
                this.treeView.Nodes.Add(this.listView.SelectedItems[0].SubItems[19].Text);

                for(int n = 20; n < 24; n++)
                {
                    this.treeView.Nodes[1].Nodes.Add(this.listView.SelectedItems[0].SubItems[n].Text);
                }
            }

            this.treeView.ExpandAll();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region IP 항목 배열 구하기 - GetIPItemArray(ipHeader, protocolType)

        /// <summary>
        /// IP 항목 배열 구하기
        /// </summary>
        /// <param name="ipHeader">IP 헤더</param>
        /// <param name="protocolType">프로토콜 타입</param>
        /// <returns>IP 항목 배열</returns>
        private string[] GetIPItemArray(IPHeader ipHeader, string protocolType)
        {
            string[] stringArray = new string[]
            {
                "IP",
                "Ver : "                    + ipHeader.Version,
                "Header Length : "          + ipHeader.HeaderLength,
                "Differntiated Services : " + ipHeader.DifferentiatedServices,
                "Total Length : "           + ipHeader.TotalLength,
                "Identification : "         + ipHeader.Identification,
                "Flags : "                  + ipHeader.Flags,
                "Fragmentation Offset : "   + ipHeader.FragmentationOffset,
                "Time to live : "           + ipHeader.TTL,
                "Protocol : "               + protocolType,
                "Checksum : "               + ipHeader.Checksum, 
                "Source : "                 + ipHeader.SourceAddress.ToString(),
                "Destination : "            + ipHeader.DestinationAddress.ToString()
            };

            return stringArray;
        }

        #endregion
        #region TCP 항목 배열 구하기 - GetTCPItemArray(tcpHeader)

        /// <summary>
        /// TCP 항목 배열 구하기
        /// </summary>
        /// <param name="tcpHeader">TCP 헤더</param>
        /// <returns>TCP 항목 배열</returns>
        private string[] GetTCPItemArray(TCPHeader tcpHeader)
        {
            string acknowledgement = null; 

            if(tcpHeader.AcknowledgementNumber != string.Empty)
            {
                acknowledgement = "Acknowledgement Number : " + tcpHeader.AcknowledgementNumber;
            }
            else 
            {
                acknowledgement =  "Acknowledgement Number : ";
            }

            string[] stringArray = new string[]
            {
                "TCP",
                "Source Port : "      + tcpHeader.SourcePort,
                "Destination Port : " + tcpHeader.DestinationPort,
                "Sequence Number : "  + tcpHeader.SequenceNumber,
                acknowledgement,
                "Header Length : "    + tcpHeader.HeaderLength,
                "Flags : "            + tcpHeader.Flags,
                "Window Size : "      + tcpHeader.WindowSize,
                "Checksum : "         + tcpHeader.Checksum};

            return stringArray;
        }

        #endregion
        #region UDP 항목 배열 구하기 - GetUDPItemArray(udpHeader)

        /// <summary>
        /// UDP 항목 배열 구하기
        /// </summary>
        /// <param name="udpHeader">UDP 헤더</param>
        /// <returns>UDP 항목 배열</returns>
        private string[] GetUDPItemArray(UDPHeader udpHeader)
        {
            string[] stringArray = new string[]
            {
                "UDP",
                "Source Port : "      + udpHeader.SourcePort,
                "Destination Port : " + udpHeader.DestinationPort,
                "Length : "           + udpHeader.Length,
                "Checksum : "         + udpHeader.Checksum
            };

            return stringArray;
        }

        #endregion

        #region 데이터 처리하기 - ProcessData(sourceByteArray, countReceived)

        /// <summary>
        /// 데이터 처리하기
        /// </summary>
        /// <param name="sourceByteArray">소스 바이트 배열</param>
        /// <param name="countReceived">수신 카운트</param>
        private void ProcessData(byte[] sourceByteArray, int countReceived)
        {
            if(this.packetNumber == Convert.ToInt32(this.packetNumberComboBox.Text))
            {
                this.packetNumber = 1;

                this.listView.Items.Clear();
            }

            IPHeader ipHeader = new IPHeader(sourceByteArray, countReceived);

            string[] baseItemArray = new string[]
            {
                this.packetNumber.ToString(),
                DateTime.Now.ToString(), 
                ipHeader.SourceAddress.ToString(),
                ipHeader.DestinationAddress.ToString(),
                ipHeader.ProtocolType.ToString(),
                ipHeader.TotalLength
            };

            string[] ipItemArray = GetIPItemArray(ipHeader, ipHeader.ProtocolType.ToString());

            string[] mergeIPItemArray = new string[baseItemArray.Length + ipItemArray.Length];

            Array.Copy(baseItemArray, 0, mergeIPItemArray, 0, baseItemArray.Length);

            Array.Copy(ipItemArray, 0, mergeIPItemArray, baseItemArray.Length, ipItemArray.Length);

            string[] mergeTCPItemArray = null;
            string[] mergeUDPItemArray = null;
            string[] allItemArray      = null;

            switch(ipHeader.ProtocolType)
            {
                case Protocol.TCP :

                    TCPHeader tcpHeader = new TCPHeader(ipHeader.DataByteArray, ipHeader.MessageLength);

                    mergeTCPItemArray = GetTCPItemArray(tcpHeader);

                    allItemArray = new string[mergeIPItemArray.Length + mergeTCPItemArray.Length];

                    Array.Copy(mergeIPItemArray, 0, allItemArray, 0, mergeIPItemArray.Length);

                    Array.Copy(mergeTCPItemArray, 0, allItemArray, mergeIPItemArray.Length, mergeTCPItemArray.Length);

                    break;

                case Protocol.UDP :

                    UDPHeader udpHeader = new UDPHeader(ipHeader.DataByteArray, (int)ipHeader.MessageLength);

                    mergeUDPItemArray = GetUDPItemArray(udpHeader);

                    allItemArray = new string[mergeIPItemArray.Length + mergeUDPItemArray.Length];

                    Array.Copy(mergeIPItemArray, 0, allItemArray, 0, mergeIPItemArray.Length);

                    Array.Copy(mergeUDPItemArray, 0, allItemArray, mergeIPItemArray.Length, mergeUDPItemArray.Length);

                    break;

                case Protocol.UNKNOWN :

                    allItemArray = mergeIPItemArray;

                    break;
            }

            ListViewItem listViewItem = new ListViewItem(allItemArray);

            this.listView.Items.Add(listViewItem);

            this.packetNumber++;
        }

        #endregion
        #region 수신 콜백 처리하기 - ReceiveCallback(asyncResult)

        /// <summary>
        /// 수신 콜백 처리하기
        /// </summary>
        /// <param name="asyncResult">비동기 결과</param>
        private void ReceiveCallback(IAsyncResult asyncResult)
        {
            try
            {
                int countReceived = this.mainSocket.EndReceive(asyncResult);

                ProcessData(this.dataByteArray, countReceived);

                if(this.continueCapturing)
                {
                    this.dataByteArray = new byte[4096];

                    this.mainSocket.BeginReceive
                    (
                        this.dataByteArray,
                        0,
                        this.dataByteArray.Length,
                        SocketFlags.None,
                        new AsyncCallback(ReceiveCallback),
                        null
                    );
                }
            }
            catch(ObjectDisposedException)
            {
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    this,
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
    }
}