﻿using System;
using System.Net;
using System.IO;

namespace TestProject
{
    /// <summary>
    /// UDP 헤더
    /// </summary>
    public class UDPHeader
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 포트
        /// </summary>
        private ushort sourcePort;

        /// <summary>
        /// 목적 포트
        /// </summary>
        private ushort destinationPort;

        /// <summary>
        /// 길이
        /// </summary>
        private ushort length;

        /// <summary>
        /// 체크썸
        /// </summary>
        private short checksum;

        /// <summary>
        /// 데이터 바이트 배열
        /// </summary>
        private byte[] dataByteArray = new byte[4096];

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 소스 포트 - SourcePort

        /// <summary>
        /// 소스 포트
        /// </summary>
        public string SourcePort
        {
            get
            {
                return this.sourcePort.ToString();
            }
        }

        #endregion
        #region 목적 포트 - DestinationPort

        /// <summary>
        /// 목적 포트
        /// </summary>
        public string DestinationPort
        {
            get
            {
                return this.destinationPort.ToString();
            }
        }

        #endregion
        #region 길이 - Length

        /// <summary>
        /// 길이
        /// </summary>
        public string Length
        {
            get
            {
                return this.length.ToString();
            }
        }

        #endregion
        #region 체크썸 - Checksum

        /// <summary>
        /// 체크썸
        /// </summary>
        public string Checksum
        {
            get
            {
                return string.Format("0x{0:x2}", this.checksum);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - UDPHeader(sourceByteArray, countReceived)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceByteArray">소스 바이트 배열</param>
        /// <param name="countReceived">수신 카운트</param>
        public UDPHeader(byte[] sourceByteArray, int countReceived)
        {
            MemoryStream memoryStream = new MemoryStream(sourceByteArray, 0, countReceived);
            BinaryReader binaryReader = new BinaryReader(memoryStream);

            this.sourcePort      = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
            this.destinationPort = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
            this.length          = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
            this.checksum        = IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());

            Array.Copy(sourceByteArray, 8, dataByteArray, 0, countReceived - 8);
        }

        #endregion
    }
}